package com.example.testktor

import android.util.Log
import androidx.compose.material3.CircularProgressIndicator
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.tooling.preview.Preview
import com.example.testktor.ui.theme.TestKtorTheme
import io.ktor.client.HttpClient
import io.ktor.client.engine.android.Android
import io.ktor.client.request.get
import io.ktor.client.statement.HttpResponse
import io.ktor.client.statement.bodyAsText
import io.ktor.http.ContentType
import io.ktor.http.contentType
import kotlinx.serialization.Serializable
import kotlinx.serialization.decodeFromString
import kotlinx.serialization.json.Json


@Serializable
data class User(val id: Int, val email: String, val password: String)

//@Composable
//fun UserScreen(modifier: Modifier = Modifier, client: HttpClient) {
//    var users by remember { mutableStateOf<List<User>?>(null) }
//    var isLoading by remember { mutableStateOf(true) }
//    var errorMessage by remember { mutableStateOf<String?>(null) }
//
//    val scope = rememberCoroutineScope()
//
//    LaunchedEffect(Unit) {
//        try {
//            val fetchedUsers = getUsers(client)
//            users = fetchedUsers
//        } catch (e: Exception) {
//            errorMessage = "Error fetching users: ${e.localizedMessage}"
//            Log.e("UserScreen", "Error: ${e.localizedMessage}", e)
//        }
//        isLoading = false
//    }
//
//    if (isLoading) {
//        CircularProgressIndicator()
//    }
//
//    errorMessage?.let {
//        Text(text = it)
//    }
//
//    users?.let {
//        it.forEach { user ->
//            Text(text = "User: ${user.name}, ID: ${user.id}")
//        }
//    } ?: run {
//        Text(text = "No users found or failed to load.")
//    }
//}
//
//suspend fun getUsers(client: HttpClient): List<User> {
//    val response: HttpResponse = client.get("http://212.67.10.54:8080/users") {
//        contentType(ContentType.Application.Json)
//    }
//    val responseBody = response.bodyAsText()
//    Log.d("getUsers", "Response: $responseBody") // Логируем ответ сервера
//
//    val json = Json { ignoreUnknownKeys = true }
//
//    return try {
//        json.decodeFromString<List<User>>(responseBody) // Десериализуем в List<User>
//    } catch (e: Exception) {
//        Log.e("getUsers", "Error deserializing response", e)
//        throw e // Повторно выбрасываем ошибку, чтобы она была обработана в UserScreen
//    }
//}
//
//@Preview(showBackground = true)
//@Composable
//fun UserScreenPreview() {
//    TestKtorTheme {
//        UserScreen(client = HttpClient(Android))
//    }
//}

@Composable
fun UserScreen(modifier: Modifier = Modifier, client: HttpClient) {
    var user by remember { mutableStateOf<User?>(null) }
    var isLoading by remember { mutableStateOf(true) }
    var errorMessage by remember { mutableStateOf<String?>(null) }

    val scope = rememberCoroutineScope()

    LaunchedEffect(Unit) {
        try {
            val fetchedUser = getUser(client) // Получаем одного пользователя
            user = fetchedUser
        } catch (e: Exception) {
            errorMessage = "Error fetching user: ${e.localizedMessage}"
            Log.e("UserScreen", "Error: ${e.localizedMessage}", e)
        }
        isLoading = false
    }

    if (isLoading) {
        CircularProgressIndicator()
    }

    errorMessage?.let {
        Text(text = it)
    }

    user?.let {
        Text(text = "User: ${it.email}, ID: ${it.id}") // Отображаем имя пользователя
    } ?: run {
        Text(text = "No user found or failed to load.")
    }
}

suspend fun getUser(client: HttpClient): User {
    val response: HttpResponse = client.get("http://10.0.2.2:8080/user") {
        contentType(ContentType.Application.Json)
    }
    val responseBody = response.bodyAsText()
    Log.d("getUser", "Response: $responseBody") // Логируем ответ сервера

    val json = Json { ignoreUnknownKeys = true }

    val buba = json.decodeFromString<User>(responseBody) // Десериализуем в объект User
    Log.d("json", buba.email)

    return try {
        json.decodeFromString<User>(responseBody) // Десериализуем в объект User
    } catch (e: Exception) {
        Log.e("getUser", "Error deserializing response", e)
        throw e // Повторно выбрасываем ошибку, чтобы она была обработана в UserScreen
    }
}

suspend fun authUser(client: HttpClient, email: String, password: String): Boolean {
    val response: HttpResponse = client.get("http://10.0.2.2:8080/user2?email=$email") {
        contentType(ContentType.Application.Json)
    }

    val responseBody = response.bodyAsText()
    Log.d("authUser", "Response: $responseBody") // Логируем ответ сервера

    val json = Json { ignoreUnknownKeys = true }

    return try {
        val user = json.decodeFromString<User>(responseBody) // Десериализуем в объект User

        // Проверяем, совпадает ли пароль
        if (user.password == password) {
            true // Возвращаем true, если пароль совпадает
        } else {
            Log.e("authUser", "Incorrect password")
            false // Возвращаем false, если пароль неверный
        }
    } catch (e: Exception) {
        Log.e("authUser", "Error deserializing response", e)
        false // Возвращаем false в случае ошибки
    }
}

@Preview(showBackground = true)
@Composable
fun UserScreenPreview() {
    TestKtorTheme {
        UserScreen(client = HttpClient(Android))
    }
}